/*
 * Author: Tomek Blaszczyk
 * e-mail: tombla@mimuw.edu.pl
 * Project: Linux Scheduler
 * Release: 1.0.0 1999.09.18
 */
#include <linux/config.h>
#include <linux/module.h>

#include <linux/sched.h>

extern int printk(const char* fmt, ...);


/*----SCHEDULING MODULE IMPLEMENTATION GOES HERE-----*/

static inline int linux_goodness(struct task_struct * p, struct task_struct * prev, int this_cpu)
{
	int weight;

	if (p->policy != SCHED_OTHER)
		return 1000 + p->rt_priority;
		
	weight = p->counter;
	if (weight>0 && p==prev) {			
		weight += 1;
	}
	return weight;
}

struct task_struct* linux_select(struct task_struct* prev, int cpu)
{
	long c;
	struct task_struct *p, *next;
	
	c = -1000L;
	next = NULL;
	p = init_task.next_run;
	while (p != &init_task) {
		int weight = linux_goodness(p,prev,cpu);
		if (weight > c)
			c = weight, next = p;
		p = p->next_run;
	}
	
	if (!c) {
		for_each_task(p) {
			p->counter = (p->counter >> 1) + p->priority;
		}
	}	
	
	return next;
}


int linux_wakeup(struct task_struct *task)
{
	if (!task->next_run && task->counter > current->counter+3)
		need_resched = 1;
		
	return 0;
}


/*
 * ---------------- MODULE-SPECIFIC functions -----------------
 */
int init_module( void )
{              
	if ( ts_installed ) {
		printk ( "linux_install: Alternate scheduler already installed.\n" );
		return -1;
	}	
	ts_installed 		= 1;
	ts_select		= linux_select;
	ts_wakeup		= linux_wakeup;
	printk ( "linux_install: OK.\n" );		
	return 0;
}

void cleanup_module(void)
{
	ts_on = 0;	
	ts_select		= NULL;
	ts_wakeup		= NULL;
	ts_installed = 0;
	printk ( "linux_uninstall: OK.\n" );
}	
