/*
 * Author: Tomek Blaszczyk
 * e-mail: tombla@mimuw.edu.pl
 * Project: Linux Scheduler
 * Release: 1.0.0 1999.09.18
 */
/****************************************************************
**
** Qt tutorial 7
**
****************************************************************/
#include <stdio.h>

#include <qapp.h>
#include <qwindow.h>
#include <qlayout.h>
#include <qlist.h>
#include <qmenubar.h>
#include <qpopmenu.h>

#include "tbschshow.hpp"
#include "tbschsrc.hpp"
#include "tbschsrcfake.hpp"
#include "tbschsrcreal.hpp"

class TBWindow : public QWindow
{
public:
	TBWindow ( QList<TBSchedSource>& srcList );
};

TBWindow::TBWindow ( QList<TBSchedSource>& srcList )
{
	setCaption ( "Scheduler - Show (L)1999 TB" );
	
	TBSchedShow* show = new TBSchedShow ( srcList, this, "show" );
	
	QBoxLayout *topLayout = new QVBoxLayout( this, 10 ); 

	//
	// Menu 
	//
	
	QMenuBar *menubar = new QMenuBar ( this );
	
	QPopupMenu *menu = new QPopupMenu();	
	menu->insertItem ( "&Run", show, SLOT(run()) );
	menu->insertItem ( "S&top", show, SLOT(stop()) );
	menu->insertItem ( "&Slower", show, SLOT(slower()) );
	menu->insertItem ( "&Faster", show, SLOT(faster()) );
	menu->insertItem ( "&Quit", qApp, SLOT(quit()) );
	
	menubar->insertItem ( "&View", menu );
	
	topLayout->setMenuBar ( menubar );
	
	topLayout->addWidget ( show, 10 );
	
	topLayout->activate();	
}


int main( int argc, char **argv )
{
	
	QList<TBSchedSource> srcList;	
	srcList.setAutoDelete(TRUE);

	int count = 0;
	for ( int k=1; k<argc; k++ )
	{
	
		int pid;
		if ( sscanf ( argv[k], "%d", &pid ) != 1 )
			break;
			
//		srcList.append ( new TBSchedSourceFake(pid) );
		srcList.append ( new TBSchedSourceReal(pid) );
		count++;
	}

	QApplication a( argc, argv );
	TBWindow w ( srcList );
	
	w.setGeometry( 50, 100, 800, 70 + count*35 );
	a.setMainWidget( &w );
	w.show();
	return a.exec();
}
