/*
 * Author: Tomek Blaszczyk
 * e-mail: tombla@mimuw.edu.pl
 * Project: Linux Scheduler
 * Release: 1.0.0 1999.09.18
 */
#include <linux/schedstat.h>
#include <linux/sched.h>

#include <string.h>

#define SCHED_CLR_STATE (0xFFFFFFFFUL>>2)
#define SCHED_CLR_TIME  (0x3UL<<(LONGBITS-2))

#define SCHED_CODE(time,event) (  ( (time) & SCHED_CLR_STATE ) | (event)  )
#define SCHED_TIME(ctime) ( (ctime) & SCHED_CLR_STATE )
#define SCHED_EVENT(ctime) ( (ctime) & SCHED_CLR_TIME )


void init_schedstat(struct schedstat_struct* sstat)
{
	if ( sstat ) {
		memset(sstat,0,sizeof(struct schedstat_struct));
	}
}


void add_event(struct schedstat_struct* sstat,
		unsigned long time,
		unsigned long event )
{
	if ( !sstat )
		return;
		
	if ( SCHED_TIME(sstat->data[sstat->index]) < SCHED_TIME(time) )
		sstat->index = (sstat->index+1) % SCHED_BUFSIZE;
	sstat->data[sstat->index]=SCHED_CODE(time,event);	
}


int get_event(struct schedstat_struct* sstat, int index,
		unsigned long* start_time,
		unsigned long* event )
{
	if ( !sstat )
		return -1;

	index %= SCHED_BUFSIZE;
	(*start_time)	= SCHED_TIME(sstat->data[index]);
	(*event)	= SCHED_EVENT(sstat->data[index]);

	if ( !(*event) )
		return -2;
		
	return 0;
}
