/*
 * Author: Tomek Blaszczyk
 * e-mail: tombla@mimuw.edu.pl
 * Project: Linux Scheduler
 * Release: 1.0.0 1999.09.18
 */
#ifndef _TBSCHSRC_H_
#define _TBSCHSRC_H_

#include <linux/schedstat.h>
#include "tbschevt.hpp"


class TBSchedSource
{

protected:
	struct schedstat_struct _sstat;
	int 		  _lastIndex;	// for looping over data
	unsigned long	  _lastTime;	// for looping over data
	bool		  _fetched;
	
private:
	unsigned long	  _currentTime;
	unsigned long 	  _endTime;
	unsigned long	  _startTime;
	int		  _pid;
	
public:
	//
	// Constructor
	//
	TBSchedSource(int pid) :
		_pid(pid),
		_startTime(0),
		_endTime(0),
		_lastIndex(0),
		_lastTime(0),
		_fetched(false)
	{
		init_schedstat(&_sstat);
	}
	
public:
	//
	// Accesors
	//
	unsigned long currentTime() { return _currentTime; }
	bool setCurrentTime(unsigned long currentTime) { _currentTime = currentTime; return true; }
	
	unsigned long endTime() { return _endTime; }
	bool setEndTime(unsigned long endTime) { _endTime = endTime; return true; }
	
	unsigned long startTime() { return _startTime; }
	bool setStartTime(unsigned long startTime) { _startTime = startTime; return true; }
	
	int pid() { return _pid; }
	bool setPid(int pid) { _pid = pid; return true; }
	
	bool fetched() { return _fetched; }

	
	//
	// Methods
	//
public:
	virtual bool FetchData(void) = 0;
	
	
	TBSchedEvent	FirstEvent(void);
	TBSchedEvent	PrevEvent(void);
};


#endif
